package blackholegame.logic;

public class BlackHoleGameLogic {

    private int rowAndColumnCount;
    private int[][] fields;
    private int actualPlayer;

    public void newGame(int rowAndColumnCount) {
        this.rowAndColumnCount = rowAndColumnCount;
        fields = new int[rowAndColumnCount][rowAndColumnCount];
        fields[rowAndColumnCount / 2][rowAndColumnCount / 2] = -1;
        for (int depth = 0; depth < rowAndColumnCount / 2; depth++) {
            fields[depth][depth] = 1;
            fields[depth][rowAndColumnCount - 1 - depth] = 1;
            fields[rowAndColumnCount - 1 - depth][depth] = 2;
            fields[rowAndColumnCount - 1 - depth][rowAndColumnCount - 1 - depth] = 2;
        }
        actualPlayer = (int) Math.round(Math.random() + 1);
    }

    public int getActualPlayer() {
        return actualPlayer;
    }

    public int getRowAndColumnCount() {
        return rowAndColumnCount;
    }

    public int getField(int row, int column) {
        return fields[row][column];
    }

    public boolean isShipOfActualPlayer(int row, int column) {
        return actualPlayer == fields[row][column];
    }

    public void move(int rowFrom, int columnFrom, int rowTo, int columnTo) {
        if (rowFrom == rowTo && columnFrom == columnTo) {
            return;
        }
        if (rowFrom == rowTo) {
            if (columnFrom < columnTo) {
                moveRight(rowFrom, columnFrom);
            } else {
                moveLeft(rowFrom, columnFrom);
            }
        } else if (columnFrom == columnTo) {
            if (rowFrom > rowTo) {
                moveUp(rowFrom, columnFrom);
            } else {
                moveDown(rowFrom, columnFrom);
            }
        }
    }

    private void moveRight(int rowFrom, int columnFrom) {
        int rowAct = rowFrom, columnAct = columnFrom;
        while (columnAct + 1 < rowAndColumnCount && fields[rowAct][columnAct + 1] == 0) {
            int field = fields[rowAct][columnAct];
            fields[rowAct][columnAct] = 0;
            fields[rowAct][columnAct + 1] = field;
            columnAct++;
        }
        if (columnAct + 1 < rowAndColumnCount && fields[rowAct][columnAct + 1] == -1) {
            fields[rowAct][columnAct] = 0;
        }
        actualPlayer = 3 - actualPlayer;
    }

    private void moveLeft(int rowFrom, int columnFrom) {
        int rowAct = rowFrom, columnAct = columnFrom;
        while (columnAct > 0 && fields[rowAct][columnAct - 1] == 0) {
            int field = fields[rowAct][columnAct];
            fields[rowAct][columnAct] = 0;
            fields[rowAct][columnAct - 1] = field;
            columnAct--;
        }
        if (columnAct > 0 && fields[rowAct][columnAct - 1] == -1) {
            fields[rowAct][columnAct] = 0;
        }
        actualPlayer = 3 - actualPlayer;
    }

    private void moveUp(int rowFrom, int columnFrom) {
        int rowAct = rowFrom, columnAct = columnFrom;
        while (rowAct > 0 && fields[rowAct - 1][columnAct] == 0) {
            int field = fields[rowAct][columnAct];
            fields[rowAct][columnAct] = 0;
            fields[rowAct - 1][columnAct] = field;
            rowAct--;
        }
        if (rowAct > 0 && fields[rowAct - 1][columnAct] == -1) {
            fields[rowAct][columnAct] = 0;
        }
        actualPlayer = 3 - actualPlayer;
    }

    private void moveDown(int rowFrom, int columnFrom) {
        int rowAct = rowFrom, columnAct = columnFrom;
        while (rowAct + 1 < rowAndColumnCount && fields[rowAct + 1][columnAct] == 0) {
            int field = fields[rowAct][columnAct];
            fields[rowAct][columnAct] = 0;
            fields[rowAct + 1][columnAct] = field;
            rowAct++;
        }
        if (rowAct + 1 < rowAndColumnCount && fields[rowAct + 1][columnAct] == -1) {
            fields[rowAct][columnAct] = 0;
        }
        actualPlayer = 3 - actualPlayer;
    }

    public boolean playerOneWon() {
        int counter = 0;
        for (int row = 0; row < rowAndColumnCount; row++) {
            for (int column = 0; column < rowAndColumnCount; column++) {
                if (fields[row][column] == 1) {
                    counter++;
                }
            }
        }
        return counter <= rowAndColumnCount / 2;
    }

    public boolean playerTwoWon() {
        int counter = 0;
        for (int row = 0; row < rowAndColumnCount; row++) {
            for (int column = 0; column < rowAndColumnCount; column++) {
                if (fields[row][column] == 2) {
                    counter++;
                }
            }
        }
        return counter <= rowAndColumnCount / 2;
    }

}
